package org.springblade.flow.form.controller;

import javax.validation.Valid;

import org.springblade.flow.base.ActionResult;
import org.springblade.flow.database.exception.DataException;
import org.springblade.flow.engine.enums.FlowStatusEnum;
import org.springblade.flow.exception.WorkFlowException;
import org.springblade.flow.form.entity.IncomeRecognitionEntity;
import org.springblade.flow.form.model.incomerecognition.IncomeRecognitionForm;
import org.springblade.flow.form.model.incomerecognition.IncomeRecognitionInfoVO;
import org.springblade.flow.form.service.IncomeRecognitionService;
import org.springblade.flow.util.JsonUtil;
import org.springblade.flow.util.JsonUtilEx;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 收入确认分析表
 *
 * @author JNPF开发平台组
 * @version V3.1.0
 * @copyright 引迈信息技术有限公司
 * @date 2019年9月27日 上午9:18
 */
@Api(tags = "收入确认分析表", value = "IncomeRecognition")
@RestController
@RequestMapping("/paas/workflow/Form/IncomeRecognition")
public class IncomeRecognitionController {

	@Autowired
	private IncomeRecognitionService incomeRecognitionService;

	/**
	 * 获取收入确认分析表信息
	 *
	 * @param id 主键值
	 * @return
	 */
	@ApiOperation("获取收入确认分析表信息")
	@GetMapping("/{id}")
	public ActionResult<IncomeRecognitionInfoVO> info(@PathVariable("id") String id) throws DataException {
		IncomeRecognitionEntity entity = incomeRecognitionService.getInfo(id);
		IncomeRecognitionInfoVO vo = JsonUtilEx.getJsonToBeanEx(entity, IncomeRecognitionInfoVO.class);
		return ActionResult.success(vo);
	}

	/**
	 * 新建收入确认分析表
	 *
	 * @param incomeRecognitionForm 表单对象
	 * @return
	 */
	@ApiOperation("新建收入确认分析表")
	@PostMapping
	public ActionResult create(@RequestBody @Valid IncomeRecognitionForm incomeRecognitionForm)
			throws WorkFlowException {
		IncomeRecognitionEntity entity = JsonUtil.getJsonToBean(incomeRecognitionForm, IncomeRecognitionEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(incomeRecognitionForm.getStatus())) {
			incomeRecognitionService.save(entity.getId(), entity);
			return ActionResult.success("保存成功");
		}
		incomeRecognitionService.submit(entity.getId(), entity);
		return ActionResult.success("提交成功，请耐心等待");
	}

	/**
	 * 修改收入确认分析表
	 *
	 * @param incomeRecognitionForm 表单对象
	 * @param id                    主键
	 * @return
	 */
	@ApiOperation("修改收入确认分析表")
	@PutMapping("/{id}")
	public ActionResult update(@RequestBody @Valid IncomeRecognitionForm incomeRecognitionForm,
			@PathVariable("id") String id) throws WorkFlowException {
		IncomeRecognitionEntity entity = JsonUtil.getJsonToBean(incomeRecognitionForm, IncomeRecognitionEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(incomeRecognitionForm.getStatus())) {
			incomeRecognitionService.save(id, entity);
			return ActionResult.success("保存成功");
		}
		incomeRecognitionService.submit(id, entity);
		return ActionResult.success("提交成功，请耐心等待");
	}
}
