package org.springblade.flow.form.controller;

import org.springblade.flow.base.ActionResult;
import org.springblade.flow.database.exception.DataException;
import org.springblade.flow.engine.enums.FlowStatusEnum;
import org.springblade.flow.exception.WorkFlowException;
import org.springblade.flow.form.entity.MonthlyReportEntity;
import org.springblade.flow.form.model.monthlyreport.MonthlyReportForm;
import org.springblade.flow.form.model.monthlyreport.MonthlyReportInfoVO;
import org.springblade.flow.form.service.MonthlyReportService;
import org.springblade.flow.util.JsonUtil;
import org.springblade.flow.util.JsonUtilEx;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 月工作总结
 *
 * @author JNPF开发平台组
 * @version V3.1.0
 * @copyright 引迈信息技术有限公司
 * @date 2019年9月27日 上午9:18
 */
@Api(tags = "月工作总结", value = "MonthlyReport")
@RestController
@RequestMapping("/paas/workflow/Form/MonthlyReport")
public class MonthlyReportController {

	@Autowired
	private MonthlyReportService monthlyReportService;

	/**
	 * 获取月工作总结信息
	 *
	 * @param id 主键值
	 * @return
	 */
	@ApiOperation("获取月工作总结信息")
	@GetMapping("/{id}")
	public ActionResult<MonthlyReportInfoVO> info(@PathVariable("id") String id) throws DataException {
		MonthlyReportEntity entity = monthlyReportService.getInfo(id);
		MonthlyReportInfoVO vo = JsonUtilEx.getJsonToBeanEx(entity, MonthlyReportInfoVO.class);
		return ActionResult.success(vo);
	}

	/**
	 * 新建月工作总结
	 *
	 * @param monthlyReportForm 表单对象
	 * @return
	 */
	@ApiOperation("新建月工作总结")
	@PostMapping
	public ActionResult create(@RequestBody MonthlyReportForm monthlyReportForm) throws WorkFlowException {
		MonthlyReportEntity entity = JsonUtil.getJsonToBean(monthlyReportForm, MonthlyReportEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(monthlyReportForm.getStatus())) {
			monthlyReportService.save(entity.getId(), entity);
			return ActionResult.success("保存成功");
		}
		monthlyReportService.submit(entity.getId(), entity);
		return ActionResult.success("提交成功，请耐心等待");
	}

	/**
	 * 修改月工作总结
	 *
	 * @param monthlyReportForm 表单对象
	 * @param id                主键
	 * @return
	 */
	@ApiOperation("修改月工作总结")
	@PutMapping("/{id}")
	public ActionResult update(@RequestBody MonthlyReportForm monthlyReportForm, @PathVariable("id") String id)
			throws WorkFlowException {
		MonthlyReportEntity entity = JsonUtil.getJsonToBean(monthlyReportForm, MonthlyReportEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(monthlyReportForm.getStatus())) {
			monthlyReportService.save(id, entity);
			return ActionResult.success("保存成功");
		}
		monthlyReportService.submit(id, entity);
		return ActionResult.success("提交成功，请耐心等待");
	}
}
