package org.springblade.flow.form.controller;

import java.util.List;

import org.springblade.flow.base.ActionResult;
import org.springblade.flow.database.exception.DataException;
import org.springblade.flow.engine.enums.FlowStatusEnum;
import org.springblade.flow.exception.WorkFlowException;
import org.springblade.flow.form.entity.OutboundEntryEntity;
import org.springblade.flow.form.entity.OutboundOrderEntity;
import org.springblade.flow.form.model.outboundorder.OutboundEntryEntityInfoModel;
import org.springblade.flow.form.model.outboundorder.OutboundOrderForm;
import org.springblade.flow.form.model.outboundorder.OutboundOrderInfoVO;
import org.springblade.flow.form.service.OutboundOrderService;
import org.springblade.flow.util.JsonUtil;
import org.springblade.flow.util.JsonUtilEx;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 出库单
 *
 * @author JNPF开发平台组
 * @version V3.1.0
 * @copyright 引迈信息技术有限公司
 * @date 2019年9月27日 上午9:18
 */
@Api(tags = "出库单", value = "OutboundOrder")
@RestController
@RequestMapping("/paas/workflow/Form/OutboundOrder")
public class OutboundOrderController {

	@Autowired
	private OutboundOrderService outboundOrderService;

	/**
	 * 获取出库单信息
	 *
	 * @param id 主键值
	 * @return
	 */
	@ApiOperation("获取出库单信息")
	@GetMapping("/{id}")
	public ActionResult<OutboundOrderInfoVO> info(@PathVariable("id") String id) throws DataException {
		OutboundOrderEntity entity = outboundOrderService.getInfo(id);
		List<OutboundEntryEntity> entityList = outboundOrderService.getOutboundEntryList(id);
		OutboundOrderInfoVO vo = JsonUtilEx.getJsonToBeanEx(entity, OutboundOrderInfoVO.class);
		vo.setEntryList(JsonUtil.getJsonToList(entityList, OutboundEntryEntityInfoModel.class));
		return ActionResult.success(vo);
	}

	/**
	 * 新建出库单
	 *
	 * @param outboundOrderForm 表单对象
	 * @return
	 * @throws WorkFlowException
	 */
	@ApiOperation("新建出库单")
	@PostMapping
	public ActionResult create(@RequestBody OutboundOrderForm outboundOrderForm) throws WorkFlowException {
		OutboundOrderEntity outbound = JsonUtil.getJsonToBean(outboundOrderForm, OutboundOrderEntity.class);
		List<OutboundEntryEntity> outboundEntryList = JsonUtil.getJsonToList(outboundOrderForm.getEntryList(),
				OutboundEntryEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(outboundOrderForm.getStatus())) {
			outboundOrderService.save(outbound.getId(), outbound, outboundEntryList);
			return ActionResult.success("保存成功");
		}
		outboundOrderService.submit(outbound.getId(), outbound, outboundEntryList);
		return ActionResult.success("提交成功，请耐心等待");
	}

	/**
	 * 修改出库单
	 *
	 * @param outboundOrderForm 表单对象
	 * @param id                主键
	 * @return
	 * @throws WorkFlowException
	 */
	@ApiOperation("修改出库单")
	@PutMapping("/{id}")
	public ActionResult update(@RequestBody OutboundOrderForm outboundOrderForm, @PathVariable("id") String id)
			throws WorkFlowException {
		OutboundOrderEntity outbound = JsonUtil.getJsonToBean(outboundOrderForm, OutboundOrderEntity.class);
		List<OutboundEntryEntity> outboundEntryList = JsonUtil.getJsonToList(outboundOrderForm.getEntryList(),
				OutboundEntryEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(outboundOrderForm.getStatus())) {
			outboundOrderService.save(id, outbound, outboundEntryList);
			return ActionResult.success("保存成功");
		}
		outboundOrderService.submit(id, outbound, outboundEntryList);
		return ActionResult.success("提交成功，请耐心等待");
	}
}
