package org.springblade.flow.form.controller;

import org.springblade.flow.base.ActionResult;
import org.springblade.flow.database.exception.DataException;
import org.springblade.flow.engine.enums.FlowStatusEnum;
import org.springblade.flow.exception.WorkFlowException;
import org.springblade.flow.form.entity.QuotationApprovalEntity;
import org.springblade.flow.form.model.quotationapproval.QuotationApprovalForm;
import org.springblade.flow.form.model.quotationapproval.QuotationApprovalInfoVO;
import org.springblade.flow.form.service.QuotationApprovalService;
import org.springblade.flow.util.JsonUtil;
import org.springblade.flow.util.JsonUtilEx;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 报价审批表
 *
 * @author JNPF开发平台组
 * @version V3.1.0
 * @copyright 引迈信息技术有限公司
 * @date 2019年9月29日 上午9:18
 */
@Api(tags = "报价审批表", value = "QuotationApproval")
@RestController
@RequestMapping("/paas/workflow/Form/QuotationApproval")
public class QuotationApprovalController {

	@Autowired
	private QuotationApprovalService quotationApprovalService;

	/**
	 * 获取报价审批表信息
	 *
	 * @param id 主键值
	 * @return
	 */
	@ApiOperation("获取报价审批表信息")
	@GetMapping("/{id}")
	public ActionResult<QuotationApprovalInfoVO> info(@PathVariable("id") String id) throws DataException {
		QuotationApprovalEntity entity = quotationApprovalService.getInfo(id);
		QuotationApprovalInfoVO vo = JsonUtilEx.getJsonToBeanEx(entity, QuotationApprovalInfoVO.class);
		return ActionResult.success(vo);
	}

	/**
	 * 新建报价审批表
	 *
	 * @param quotationApprovalForm 表单对象
	 * @return
	 */
	@ApiOperation("新建报价审批表")
	@PostMapping
	public ActionResult create(@RequestBody QuotationApprovalForm quotationApprovalForm) throws WorkFlowException {
		QuotationApprovalEntity entity = JsonUtil.getJsonToBean(quotationApprovalForm, QuotationApprovalEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(quotationApprovalForm.getStatus())) {
			quotationApprovalService.save(entity.getId(), entity);
			return ActionResult.success("保存成功");
		}
		quotationApprovalService.submit(entity.getId(), entity);
		return ActionResult.success("提交成功，请耐心等待");
	}

	/**
	 * 修改报价审批表
	 *
	 * @param quotationApprovalForm 表单对象
	 * @param id                    主键
	 * @return
	 */
	@ApiOperation("修改报价审批表")
	@PutMapping("/{id}")
	public ActionResult update(@RequestBody QuotationApprovalForm quotationApprovalForm, @PathVariable("id") String id)
			throws WorkFlowException {
		QuotationApprovalEntity entity = JsonUtil.getJsonToBean(quotationApprovalForm, QuotationApprovalEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(quotationApprovalForm.getStatus())) {
			quotationApprovalService.save(id, entity);
			return ActionResult.success("保存成功");
		}
		quotationApprovalService.submit(id, entity);
		return ActionResult.success("提交成功，请耐心等待");
	}
}
