package org.springblade.flow.form.controller;

import org.springblade.flow.base.ActionResult;
import org.springblade.flow.database.exception.DataException;
import org.springblade.flow.engine.enums.FlowStatusEnum;
import org.springblade.flow.exception.WorkFlowException;
import org.springblade.flow.form.entity.ReceiptProcessingEntity;
import org.springblade.flow.form.model.receiptprocessing.ReceiptProcessingForm;
import org.springblade.flow.form.model.receiptprocessing.ReceiptProcessingInfoVO;
import org.springblade.flow.form.service.ReceiptProcessingService;
import org.springblade.flow.util.JsonUtil;
import org.springblade.flow.util.JsonUtilEx;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 收文处理表
 *
 * @author JNPF开发平台组
 * @version V3.1.0
 * @copyright 引迈信息技术有限公司
 * @date 2019年9月29日 上午9:18
 */
@Api(tags = "收文处理表", value = "ReceiptProcessing")
@RestController
@RequestMapping("/paas/workflow/Form/ReceiptProcessing")
public class ReceiptProcessingController {

	@Autowired
	private ReceiptProcessingService receiptProcessingService;

	/**
	 * 获取收文处理表信息
	 *
	 * @param id 主键值
	 * @return
	 */
	@ApiOperation("获取收文处理表信息")
	@GetMapping("/{id}")
	public ActionResult<ReceiptProcessingInfoVO> info(@PathVariable("id") String id) throws DataException {
		ReceiptProcessingEntity entity = receiptProcessingService.getInfo(id);
		ReceiptProcessingInfoVO vo = JsonUtilEx.getJsonToBeanEx(entity, ReceiptProcessingInfoVO.class);
		return ActionResult.success(vo);
	}

	/**
	 * 新建收文处理表
	 *
	 * @param receiptProcessingForm 表单对象
	 * @return
	 */
	@ApiOperation("新建收文处理表")
	@PostMapping
	public ActionResult create(@RequestBody ReceiptProcessingForm receiptProcessingForm) throws WorkFlowException {
		ReceiptProcessingEntity entity = JsonUtil.getJsonToBean(receiptProcessingForm, ReceiptProcessingEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(receiptProcessingForm.getStatus())) {
			receiptProcessingService.save(entity.getId(), entity);
			return ActionResult.success("保存成功");
		}
		receiptProcessingService.submit(entity.getId(), entity);
		return ActionResult.success("提交成功，请耐心等待");
	}

	/**
	 * 修改收文处理表
	 *
	 * @param receiptProcessingForm 表单对象
	 * @param id                    主键
	 * @return
	 */
	@ApiOperation("修改收文处理表")
	@PutMapping("/{id}")
	public ActionResult update(@RequestBody ReceiptProcessingForm receiptProcessingForm, @PathVariable("id") String id)
			throws WorkFlowException {
		ReceiptProcessingEntity entity = JsonUtil.getJsonToBean(receiptProcessingForm, ReceiptProcessingEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(receiptProcessingForm.getStatus())) {
			receiptProcessingService.save(id, entity);
			return ActionResult.success("保存成功");
		}
		receiptProcessingService.submit(id, entity);
		return ActionResult.success("提交成功，请耐心等待");
	}
}
