package org.springblade.flow.form.controller;

import org.springblade.flow.base.ActionResult;
import org.springblade.flow.database.exception.DataException;
import org.springblade.flow.engine.enums.FlowStatusEnum;
import org.springblade.flow.exception.WorkFlowException;
import org.springblade.flow.form.entity.ReceiptSignEntity;
import org.springblade.flow.form.model.receiptsign.ReceiptSignForm;
import org.springblade.flow.form.model.receiptsign.ReceiptSignInfoVO;
import org.springblade.flow.form.service.ReceiptSignService;
import org.springblade.flow.util.JsonUtil;
import org.springblade.flow.util.JsonUtilEx;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 收文签呈单
 *
 * @author JNPF开发平台组
 * @version V3.1.0
 * @copyright 引迈信息技术有限公司
 * @date 2019年9月29日 上午9:18
 */
@Api(tags = "收文签呈单", value = "ReceiptSign")
@RestController
@RequestMapping("/paas/workflow/Form/ReceiptSign")
public class ReceiptSignController {

	@Autowired
	private ReceiptSignService receiptSignService;

	/**
	 * 获取收文签呈单信息
	 *
	 * @param id 主键值
	 * @return
	 */
	@ApiOperation("获取收文签呈单信息")
	@GetMapping("/{id}")
	public ActionResult<ReceiptSignInfoVO> info(@PathVariable("id") String id) throws DataException {
		ReceiptSignEntity entity = receiptSignService.getInfo(id);
		ReceiptSignInfoVO vo = JsonUtilEx.getJsonToBeanEx(entity, ReceiptSignInfoVO.class);
		return ActionResult.success(vo);
	}

	/**
	 * 新建收文签呈单
	 *
	 * @param receiptSignForm 表单对象
	 * @return
	 */
	@ApiOperation("新建收文签呈单")
	@PostMapping
	public ActionResult create(@RequestBody ReceiptSignForm receiptSignForm) throws WorkFlowException {
		ReceiptSignEntity entity = JsonUtil.getJsonToBean(receiptSignForm, ReceiptSignEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(receiptSignForm.getStatus())) {
			receiptSignService.save(entity.getId(), entity);
			return ActionResult.success("保存成功");
		}
		receiptSignService.submit(entity.getId(), entity);
		return ActionResult.success("提交成功，请耐心等待");
	}

	/**
	 * 修改收文签呈单
	 *
	 * @param receiptSignForm 表单对象
	 * @param id              主键
	 * @return
	 */
	@ApiOperation("修改收文签呈单")
	@PutMapping("/{id}")
	public ActionResult update(@RequestBody ReceiptSignForm receiptSignForm, @PathVariable("id") String id)
			throws WorkFlowException {
		ReceiptSignEntity entity = JsonUtil.getJsonToBean(receiptSignForm, ReceiptSignEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(receiptSignForm.getStatus())) {
			receiptSignService.save(id, entity);
			return ActionResult.success("保存成功");
		}
		receiptSignService.submit(id, entity);
		return ActionResult.success("提交成功，请耐心等待");
	}
}
