package org.springblade.flow.form.controller;

import org.springblade.flow.base.ActionResult;
import org.springblade.flow.database.exception.DataException;
import org.springblade.flow.engine.enums.FlowStatusEnum;
import org.springblade.flow.exception.WorkFlowException;
import org.springblade.flow.form.entity.TravelReimbursementEntity;
import org.springblade.flow.form.model.travelreimbursement.TravelReimbursementForm;
import org.springblade.flow.form.model.travelreimbursement.TravelReimbursementInfoVO;
import org.springblade.flow.form.service.TravelReimbursementService;
import org.springblade.flow.util.JsonUtil;
import org.springblade.flow.util.JsonUtilEx;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 差旅报销申请表
 *
 * @author JNPF开发平台组
 * @version V3.1.0
 * @copyright 引迈信息技术有限公司
 * @date 2019年9月29日 上午9:18
 */
@Api(tags = "差旅报销申请表", value = "TravelReimbursement")
@RestController
@RequestMapping("/paas/workflow/Form/TravelReimbursement")
public class TravelReimbursementController {

	@Autowired
	private TravelReimbursementService travelReimbursementService;

	/**
	 * 获取差旅报销申请表信息
	 *
	 * @param id 主键值
	 * @return
	 */
	@ApiOperation("获取差旅报销申请表信息")
	@GetMapping("/{id}")
	public ActionResult<TravelReimbursementInfoVO> info(@PathVariable("id") String id) throws DataException {
		TravelReimbursementEntity entity = travelReimbursementService.getInfo(id);
		TravelReimbursementInfoVO vo = JsonUtilEx.getJsonToBeanEx(entity, TravelReimbursementInfoVO.class);
		return ActionResult.success(vo);
	}

	/**
	 * 新建差旅报销申请表
	 *
	 * @param travelReimbursementForm 表单对象
	 * @return
	 */
	@ApiOperation("新建差旅报销申请表")
	@PostMapping
	public ActionResult create(@RequestBody TravelReimbursementForm travelReimbursementForm) throws WorkFlowException {
		if (travelReimbursementForm.getSetOutDate() > travelReimbursementForm.getReturnDate()) {
			return ActionResult.fail("结束时间不能小于起始时间");
		}
		TravelReimbursementEntity entity = JsonUtil.getJsonToBean(travelReimbursementForm,
				TravelReimbursementEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(travelReimbursementForm.getStatus())) {
			travelReimbursementService.save(entity.getId(), entity);
			return ActionResult.success("保存成功");
		}
		travelReimbursementService.submit(entity.getId(), entity);
		return ActionResult.success("提交成功，请耐心等待");
	}

	/**
	 * 修改差旅报销申请表
	 *
	 * @param travelReimbursementForm 表单对象
	 * @param id                      主键
	 * @return
	 */
	@ApiOperation("修改差旅报销申请表")
	@PutMapping("/{id}")
	public ActionResult update(@RequestBody TravelReimbursementForm travelReimbursementForm,
			@PathVariable("id") String id) throws WorkFlowException {
		if (travelReimbursementForm.getSetOutDate() > travelReimbursementForm.getReturnDate()) {
			return ActionResult.fail("结束时间不能小于起始时间");
		}
		TravelReimbursementEntity entity = JsonUtil.getJsonToBean(travelReimbursementForm,
				TravelReimbursementEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(travelReimbursementForm.getStatus())) {
			travelReimbursementService.save(id, entity);
			return ActionResult.success("保存成功");
		}
		travelReimbursementService.submit(id, entity);
		return ActionResult.success("提交成功，请耐心等待");
	}
}
