package org.springblade.flow.form.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.springblade.flow.base.service.BillRuleService;
import org.springblade.flow.base.util.FileManageUtil;
import org.springblade.flow.engine.util.ModelUtil;
import org.springblade.flow.exception.WorkFlowException;
import org.springblade.flow.form.entity.SalesOrderEntity;
import org.springblade.flow.form.entity.SalesOrderEntryEntity;
import org.springblade.flow.form.mapper.SalesOrderMapper;
import org.springblade.flow.form.model.salesorder.SalesOrderEntryEntityInfoModel;
import org.springblade.flow.form.model.salesorder.SalesOrderForm;
import org.springblade.flow.form.service.SalesOrderEntryService;
import org.springblade.flow.form.service.SalesOrderService;
import org.springblade.flow.model.FileModel;
import org.springblade.flow.util.JsonUtil;
import org.springblade.flow.util.RandomUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

/**
 * 销售订单
 *
 * @author JNPF开发平台组
 * @version V3.1.0
 * @copyright 引迈信息技术有限公司
 * @date 2019年9月29日 上午9:18
 */
@Service
public class SalesOrderServiceImpl extends ServiceImpl<SalesOrderMapper, SalesOrderEntity>
		implements SalesOrderService {

	@Autowired
	private BillRuleService billRuleService;
	@Autowired
	private SalesOrderEntryService salesOrderEntryService;
	@Autowired
	private FileManageUtil fileManageUtil;

	@Override
	public List<SalesOrderEntryEntity> getSalesEntryList(String id) {
		QueryWrapper<SalesOrderEntryEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(SalesOrderEntryEntity::getSalesOrderId, id)
				.orderByDesc(SalesOrderEntryEntity::getSortCode);
		return salesOrderEntryService.list(queryWrapper);
	}

	@Override
	public SalesOrderEntity getInfo(String id) {
		QueryWrapper<SalesOrderEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(SalesOrderEntity::getId, id);
		return this.getOne(queryWrapper);
	}

	@Override
	@Transactional(rollbackFor = WorkFlowException.class)
	public void save(String id, SalesOrderEntity entity, List<SalesOrderEntryEntity> salesOrderEntryEntityList)
			throws WorkFlowException {
		// 表单信息
		if (id == null) {
			entity.setId(RandomUtil.uuId());
			for (int i = 0; i < salesOrderEntryEntityList.size(); i++) {
				salesOrderEntryEntityList.get(i).setId(RandomUtil.uuId());
				salesOrderEntryEntityList.get(i).setSalesOrderId(entity.getId());
				salesOrderEntryEntityList.get(i).setSortCode(Long.parseLong(i + ""));
				salesOrderEntryService.save(salesOrderEntryEntityList.get(i));
			}
			// 创建
			this.save(entity);
			billRuleService.useBillNumber("WF_SalesOrderNo");
			// 添加附件
			List<FileModel> data = JsonUtil.getJsonToList(entity.getFileJson(), FileModel.class);
			fileManageUtil.createFile(data);
		} else {
			entity.setId(id);
			QueryWrapper<SalesOrderEntryEntity> queryWrapper = new QueryWrapper<>();
			queryWrapper.lambda().eq(SalesOrderEntryEntity::getSalesOrderId, entity.getId());
			salesOrderEntryService.remove(queryWrapper);
			for (int i = 0; i < salesOrderEntryEntityList.size(); i++) {
				salesOrderEntryEntityList.get(i).setId(RandomUtil.uuId());
				salesOrderEntryEntityList.get(i).setSalesOrderId(entity.getId());
				salesOrderEntryEntityList.get(i).setSortCode(Long.parseLong(i + ""));
				salesOrderEntryService.save(salesOrderEntryEntityList.get(i));
			}
			// 编辑
			this.updateById(entity);
			// 更新附件
			List<FileModel> data = JsonUtil.getJsonToList(entity.getFileJson(), FileModel.class);
			fileManageUtil.updateFile(data);
		}
		// 流程信息
		ModelUtil.save(id, entity.getFlowId(), entity.getId(), entity.getFlowTitle(), entity.getFlowUrgent(),
				entity.getBillNo(), entity);
	}

	@Override
	@Transactional(rollbackFor = WorkFlowException.class)
	public void submit(String id, SalesOrderEntity entity, List<SalesOrderEntryEntity> salesOrderEntryEntityList)
			throws WorkFlowException {
		// 表单信息
		if (id == null) {
			entity.setId(RandomUtil.uuId());
			for (int i = 0; i < salesOrderEntryEntityList.size(); i++) {
				salesOrderEntryEntityList.get(i).setId(RandomUtil.uuId());
				salesOrderEntryEntityList.get(i).setSalesOrderId(entity.getId());
				salesOrderEntryEntityList.get(i).setSortCode(Long.parseLong(i + ""));
				salesOrderEntryService.save(salesOrderEntryEntityList.get(i));
			}
			// 创建
			this.save(entity);
			billRuleService.useBillNumber("WF_SalesOrderNo");
			// 添加附件
			List<FileModel> data = JsonUtil.getJsonToList(entity.getFileJson(), FileModel.class);
			fileManageUtil.createFile(data);
		} else {
			entity.setId(id);
			QueryWrapper<SalesOrderEntryEntity> queryWrapper = new QueryWrapper<>();
			queryWrapper.lambda().eq(SalesOrderEntryEntity::getSalesOrderId, entity.getId());
			salesOrderEntryService.remove(queryWrapper);
			for (int i = 0; i < salesOrderEntryEntityList.size(); i++) {
				salesOrderEntryEntityList.get(i).setId(RandomUtil.uuId());
				salesOrderEntryEntityList.get(i).setSalesOrderId(entity.getId());
				salesOrderEntryEntityList.get(i).setSortCode(Long.parseLong(i + ""));
				salesOrderEntryService.save(salesOrderEntryEntityList.get(i));
			}
			// 编辑
			this.updateById(entity);
			// 更新附件
			List<FileModel> data = JsonUtil.getJsonToList(entity.getFileJson(), FileModel.class);
			fileManageUtil.updateFile(data);
		}
		// 流程信息
		ModelUtil.submit(id, entity.getFlowId(), entity.getId(), entity.getFlowTitle(), entity.getFlowUrgent(),
				entity.getBillNo(), entity, null);
	}

	@Override
	public void data(String id, String data) {
		SalesOrderForm salesOrderForm = JsonUtil.getJsonToBean(data, SalesOrderForm.class);
		SalesOrderEntity entity = JsonUtil.getJsonToBean(salesOrderForm, SalesOrderEntity.class);
		List<SalesOrderEntryEntityInfoModel> entryList = salesOrderForm.getEntryList() != null
				? salesOrderForm.getEntryList()
				: new ArrayList<>();
		List<SalesOrderEntryEntity> salesOrderEntryEntityList = JsonUtil.getJsonToList(entryList,
				SalesOrderEntryEntity.class);
		entity.setId(id);
		QueryWrapper<SalesOrderEntryEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(SalesOrderEntryEntity::getSalesOrderId, entity.getId());
		salesOrderEntryService.remove(queryWrapper);
		for (int i = 0; i < salesOrderEntryEntityList.size(); i++) {
			salesOrderEntryEntityList.get(i).setId(RandomUtil.uuId());
			salesOrderEntryEntityList.get(i).setSalesOrderId(entity.getId());
			salesOrderEntryEntityList.get(i).setSortCode(Long.parseLong(i + ""));
			salesOrderEntryService.save(salesOrderEntryEntityList.get(i));
		}
		// 编辑
		this.saveOrUpdate(entity);
	}
}
